@@ -486,8 +486,7 @@ static void storeLoopPkt (WVIEWD_WORK *work, LOOP_PKT *dest, TE923_DATA *src)
     if (0 <= src->windAvgSpeed && src->windAvgSpeed <= 250)
     {
         tempfloat = src->windAvgSpeed;
-        tempfloat += 0.5;
-        dest->windSpeed  = (uint16_t)tempfloat;
+        dest->windSpeedF     = tempfloat;
     }
 
     if (0 <= src->windDir && src->windDir <= 360)
@@ -501,8 +500,7 @@ static void storeLoopPkt (WVIEWD_WORK *work, LOOP_PKT *dest, TE923_DATA *src)
     if (0 <= src->windGustSpeed && src->windGustSpeed <= 250)
     {
         tempfloat = src->windGustSpeed;
-        tempfloat += 0.5;
-        dest->windGust       = (uint16_t)tempfloat;
+        dest->windGustF      = tempfloat;
     }
 
     if (0 <= src->rain)
